@@ -3,7 +3,7 @@
  **********************************************************************
 
   ga_chromo - Genetic algorithm chromosome handling routines.
-  Copyright ©2000-2003, Stewart Adcock <stewart@linux-domain.com>
+  Copyright ©2000-2006, Stewart Adcock <stewart@linux-domain.com>
   All rights reserved.
 
   The latest version of this program should be available at:
@@ -37,6 +37,14 @@
 		ga_chromosome_XXX_to_string() - Human readable NULL-
 		   terminated string.
 
+		The chromosome types are:
+		integer - C int.
+		boolean - boolean (generally, C _Bool by default).
+		double - C double.
+		char - C char.
+		bitstring - bitstring.
+		list - generic linked-list.
+
   To do:	Will need chromosome comparison functions.
 
  **********************************************************************/
@@ -49,10 +57,10 @@
 		contents are garbage (there is no need to zero them).
   parameters:
   return:
-  last updated: 13/06/01
+  last updated: 05 Oct 2004
  **********************************************************************/
 
-boolean ga_chromosome_integer_allocate( const population *pop,
+GAULFUNC boolean ga_chromosome_integer_allocate( population *pop,
                                         entity *embryo )
   {
   int		i;		/* Loop variable over all chromosomes */
@@ -63,8 +71,23 @@ boolean ga_chromosome_integer_allocate( const population *pop,
   if (embryo->chromosome!=NULL)
     die("This entity already contains chromosomes.");
 
-  embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(int *));
-  embryo->chromosome[0] = s_malloc(pop->num_chromosomes*pop->len_chromosomes*sizeof(int));
+#ifdef USE_CHROMO_CHUNKS
+  THREAD_LOCK(pop->chromo_chunk_lock);
+  if (!pop->chromo_chunk)
+    {
+    pop->chromoarray_chunk = mem_chunk_new(pop->num_chromosomes*sizeof(int *), 1024);
+    pop->chromo_chunk = mem_chunk_new(pop->num_chromosomes*pop->len_chromosomes*sizeof(int), 2048); 
+    }
+
+  embryo->chromosome = mem_chunk_alloc(pop->chromoarray_chunk);
+  embryo->chromosome[0] = mem_chunk_alloc(pop->chromo_chunk);
+  THREAD_UNLOCK(pop->chromo_chunk_lock);
+#else
+  if ( !(embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(int *))) )
+    die("Unable to allocate memory");
+  if ( !(embryo->chromosome[0] = s_malloc(pop->num_chromosomes*pop->len_chromosomes*sizeof(int))) )
+    die("Unable to allocate memory");
+#endif
 
   for (i=1; i<pop->num_chromosomes; i++)
     {
@@ -77,13 +100,13 @@ boolean ga_chromosome_integer_allocate( const population *pop,
 
 /**********************************************************************
   ga_chromosome_integer_deallocate()
-  synopsis:	Dellocate the chromosomes for an entity.
+  synopsis:	Deallocate the chromosomes for an entity.
   parameters:
   return:
-  last updated: 13/06/01
+  last updated: 05 Oct 2004
  **********************************************************************/
 
-void ga_chromosome_integer_deallocate( const population *pop,
+GAULFUNC void ga_chromosome_integer_deallocate( population *pop,
                                        entity *corpse )
   {
 
@@ -93,9 +116,24 @@ void ga_chromosome_integer_deallocate( const population *pop,
   if (corpse->chromosome==NULL)
     die("This entity already contains no chromosomes.");
 
+#ifdef USE_CHROMO_CHUNKS
+  THREAD_LOCK(pop->chromo_chunk_lock);
+  mem_chunk_free(pop->chromo_chunk, corpse->chromosome[0]);
+  mem_chunk_free(pop->chromoarray_chunk, corpse->chromosome);
+  corpse->chromosome=NULL;
+
+  if (mem_chunk_isempty(pop->chromo_chunk))
+    {
+    mem_chunk_destroy(pop->chromo_chunk);
+    mem_chunk_destroy(pop->chromoarray_chunk);
+    pop->chromo_chunk = NULL;
+    }
+  THREAD_UNLOCK(pop->chromo_chunk_lock);
+#else
   s_free(corpse->chromosome[0]);
   s_free(corpse->chromosome);
   corpse->chromosome=NULL;
+#endif
 
   return;
   }
@@ -109,7 +147,7 @@ void ga_chromosome_integer_deallocate( const population *pop,
   last updated: 13/06/01
  **********************************************************************/
 
-void ga_chromosome_integer_replicate( const population *pop,
+GAULFUNC void ga_chromosome_integer_replicate( const population *pop,
                                       entity *parent, entity *child,
                                       const int chromosomeid )
   {
@@ -135,8 +173,8 @@ void ga_chromosome_integer_replicate( const population *pop,
   last updated: 1 Feb 2002
  **********************************************************************/
 
-unsigned int ga_chromosome_integer_to_bytes(const population *pop, entity *joe,
-                                     byte **bytes, unsigned int *max_bytes)
+GAULFUNC unsigned int ga_chromosome_integer_to_bytes(const population *pop, entity *joe,
+                                     gaulbyte **bytes, unsigned int *max_bytes)
   {
   int		num_bytes;	/* Actual size of genes. */
 
@@ -147,14 +185,14 @@ unsigned int ga_chromosome_integer_to_bytes(const population *pop, entity *joe,
 
   if (!joe->chromosome)
     {
-    *bytes = (byte *)"\0";
+    *bytes = (gaulbyte *)"\0";
     return 0;
     }
 
   num_bytes = pop->len_chromosomes * pop->num_chromosomes *
               sizeof(int);
 
-  *bytes = (byte *)joe->chromosome[0];
+  *bytes = (gaulbyte *)joe->chromosome[0];
 
   return num_bytes;
   }
@@ -169,7 +207,7 @@ unsigned int ga_chromosome_integer_to_bytes(const population *pop, entity *joe,
   last updated: 1 Feb 2002
  **********************************************************************/
 
-void ga_chromosome_integer_from_bytes(const population *pop, entity *joe, byte *bytes)
+GAULFUNC void ga_chromosome_integer_from_bytes(const population *pop, entity *joe, gaulbyte *bytes)
   {
 
   if (!pop) die("Null pointer to population structure passed.");
@@ -195,7 +233,7 @@ void ga_chromosome_integer_from_bytes(const population *pop, entity *joe, byte *
   last updated: 19 Aug 2002
  **********************************************************************/
 
-char *ga_chromosome_integer_to_string(
+GAULFUNC char *ga_chromosome_integer_to_string(
                               const population *pop, const entity *joe,
                               char *text, size_t *textlen)
   {
@@ -207,7 +245,7 @@ char *ga_chromosome_integer_to_string(
   if (!joe) die("Null pointer to entity structure passed.");
 
 /* Ensure that a reasonable amount of memory is allocated. */
-  if (!text || *textlen < 8 * pop->len_chromosomes * pop->num_chromosomes)
+  if (!text || (int) *textlen < 8 * pop->len_chromosomes * pop->num_chromosomes)
     {
     *textlen = 8 * pop->len_chromosomes * pop->num_chromosomes;
     text = s_realloc(text, sizeof(char) * *textlen);
@@ -226,7 +264,7 @@ char *ga_chromosome_integer_to_string(
       {
       if (*textlen-k<8)
         {
-        *textlen *= 2;   /* FIXME: This isn't intelligent. */
+        *textlen *= 2;   /* Double allocation. */
         text = s_realloc(text, sizeof(char) * *textlen);
         }
 
@@ -235,7 +273,7 @@ char *ga_chromosome_integer_to_string(
 
       if (l == -1)
         {	/* Truncation occured. */
-	*textlen *= 2;	/* FIXME: This isn't intelligent. */
+	*textlen *= 2;	/* Double allocation. */
         text = s_realloc(text, sizeof(char) * *textlen);
         l = snprintf(&(text[k]), *textlen-k, "%d ",
                        ((int *)joe->chromosome[i])[j]);
@@ -263,7 +301,7 @@ char *ga_chromosome_integer_to_string(
   last updated: 13/06/01
  **********************************************************************/
 
-boolean ga_chromosome_boolean_allocate(const population *pop, entity *embryo)
+GAULFUNC boolean ga_chromosome_boolean_allocate(population *pop, entity *embryo)
   {
   int		i;		/* Loop variable over all chromosomes */
 
@@ -273,8 +311,10 @@ boolean ga_chromosome_boolean_allocate(const population *pop, entity *embryo)
   if (embryo->chromosome!=NULL)
     die("This entity already contains chromosomes.");
 
-  embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(boolean *));
-  embryo->chromosome[0] = s_malloc(pop->num_chromosomes*pop->len_chromosomes*sizeof(boolean));
+  if ( !(embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(boolean *))) )
+    die("Unable to allocate memory");
+  if ( !(embryo->chromosome[0] = s_malloc(pop->num_chromosomes*pop->len_chromosomes*sizeof(boolean))) )
+    die("Unable to allocate memory");
 
   for (i=1; i<pop->num_chromosomes; i++)
     {
@@ -287,13 +327,13 @@ boolean ga_chromosome_boolean_allocate(const population *pop, entity *embryo)
 
 /**********************************************************************
   ga_chromosome_boolean_deallocate()
-  synopsis:	Dellocate the chromosomes for an entity.
+  synopsis:	Deallocate the chromosomes for an entity.
   parameters:
   return:
   last updated: 13/06/01
  **********************************************************************/
 
-void ga_chromosome_boolean_deallocate(const population *pop, entity *corpse)
+GAULFUNC void ga_chromosome_boolean_deallocate(population *pop, entity *corpse)
   {
 
   if (!pop) die("Null pointer to population structure passed.");
@@ -318,7 +358,7 @@ void ga_chromosome_boolean_deallocate(const population *pop, entity *corpse)
   last updated: 19 Mar 2002
  **********************************************************************/
 
-void ga_chromosome_boolean_replicate( const population *pop,
+GAULFUNC void ga_chromosome_boolean_replicate( const population *pop,
                                       entity *parent, entity *child,
                                       const int chromosomeid )
   {
@@ -345,8 +385,8 @@ void ga_chromosome_boolean_replicate( const population *pop,
   last updated: 1 Feb 2002
  **********************************************************************/
 
-unsigned int ga_chromosome_boolean_to_bytes(const population *pop, entity *joe,
-                                    byte **bytes, unsigned int *max_bytes)
+GAULFUNC unsigned int ga_chromosome_boolean_to_bytes(const population *pop, entity *joe,
+                                    gaulbyte **bytes, unsigned int *max_bytes)
   {
   int		num_bytes;	/* Actual size of genes. */
 
@@ -357,14 +397,14 @@ unsigned int ga_chromosome_boolean_to_bytes(const population *pop, entity *joe,
 
   if (!joe->chromosome)
     {
-    *bytes = (byte *)"\0";
+    *bytes = (gaulbyte *)"\0";
     return 0;
     }
 
   num_bytes = pop->len_chromosomes * pop->num_chromosomes *
               sizeof(boolean);
 
-  *bytes = (byte *)joe->chromosome[0];
+  *bytes = (gaulbyte *)joe->chromosome[0];
 
   return num_bytes;
   }
@@ -379,7 +419,7 @@ unsigned int ga_chromosome_boolean_to_bytes(const population *pop, entity *joe,
   last updated: 1 Feb 2002
  **********************************************************************/
 
-void ga_chromosome_boolean_from_bytes(const population *pop, entity *joe, byte *bytes)
+GAULFUNC void ga_chromosome_boolean_from_bytes(const population *pop, entity *joe, gaulbyte *bytes)
   {
 
   if (!pop) die("Null pointer to population structure passed.");
@@ -402,7 +442,7 @@ void ga_chromosome_boolean_from_bytes(const population *pop, entity *joe, byte *
   last updated: 19 Aug 2002
  **********************************************************************/
 
-char *ga_chromosome_boolean_to_string(
+GAULFUNC char *ga_chromosome_boolean_to_string(
                               const population *pop, const entity *joe,
                               char *text, size_t *textlen)
   {
@@ -412,7 +452,7 @@ char *ga_chromosome_boolean_to_string(
   if (!pop) die("Null pointer to population structure passed.");
   if (!joe) die("Null pointer to entity structure passed.");
 
-  if (!text || *textlen < pop->len_chromosomes * pop->num_chromosomes + 1)
+  if (!text || (int) *textlen < pop->len_chromosomes * pop->num_chromosomes + 1)
     {
     *textlen = pop->len_chromosomes * pop->num_chromosomes + 1;
     text = s_realloc(text, sizeof(char) * *textlen);
@@ -447,7 +487,7 @@ char *ga_chromosome_boolean_to_string(
   last updated: 16/06/01
  **********************************************************************/
 
-boolean ga_chromosome_double_allocate(const population *pop, entity *embryo)
+GAULFUNC boolean ga_chromosome_double_allocate(population *pop, entity *embryo)
   {
   int		i;		/* Loop variable over all chromosomes */
 
@@ -457,8 +497,10 @@ boolean ga_chromosome_double_allocate(const population *pop, entity *embryo)
   if (embryo->chromosome!=NULL)
     die("This entity already contains chromosomes.");
 
-  embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(double *));
-  embryo->chromosome[0] = s_malloc(pop->num_chromosomes*pop->len_chromosomes*sizeof(double));
+  if ( !(embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(double *))) )
+    die("Unable to allocate memory");
+  if ( !(embryo->chromosome[0] = s_malloc(pop->num_chromosomes*pop->len_chromosomes*sizeof(double))) )
+    die("Unable to allocate memory");
 
   for (i=1; i<pop->num_chromosomes; i++)
     {
@@ -471,13 +513,13 @@ boolean ga_chromosome_double_allocate(const population *pop, entity *embryo)
 
 /**********************************************************************
   ga_chromosome_double_deallocate()
-  synopsis:	Dellocate the chromosomes for an entity.
+  synopsis:	Deallocate the chromosomes for an entity.
   parameters:
   return:
   last updated: 16/06/01
  **********************************************************************/
 
-void ga_chromosome_double_deallocate(const population *pop, entity *corpse)
+GAULFUNC void ga_chromosome_double_deallocate(population *pop, entity *corpse)
   {
 
   if (!pop) die("Null pointer to population structure passed.");
@@ -502,7 +544,7 @@ void ga_chromosome_double_deallocate(const population *pop, entity *corpse)
   last updated: 16/06/01
  **********************************************************************/
 
-void ga_chromosome_double_replicate( const population *pop,
+GAULFUNC void ga_chromosome_double_replicate( const population *pop,
                                       entity *parent, entity *child,
                                       const int chromosomeid )
   {
@@ -529,8 +571,8 @@ void ga_chromosome_double_replicate( const population *pop,
   last updated: 1 Feb 2002
  **********************************************************************/
 
-unsigned int ga_chromosome_double_to_bytes(const population *pop, entity *joe,
-                                    byte **bytes, unsigned int *max_bytes)
+GAULFUNC unsigned int ga_chromosome_double_to_bytes(const population *pop, entity *joe,
+                                    gaulbyte **bytes, unsigned int *max_bytes)
   {
   int		num_bytes;	/* Actual size of genes. */
 
@@ -541,14 +583,14 @@ unsigned int ga_chromosome_double_to_bytes(const population *pop, entity *joe,
 
   if (!joe->chromosome)
     {
-    *bytes = (byte *)"\0";
+    *bytes = (gaulbyte *)"\0";
     return 0;
     }
 
   num_bytes = pop->len_chromosomes * pop->num_chromosomes *
               sizeof(double);
 
-  *bytes = (byte *)joe->chromosome[0];
+  *bytes = (gaulbyte *)joe->chromosome[0];
 
   return num_bytes;
   }
@@ -563,7 +605,7 @@ unsigned int ga_chromosome_double_to_bytes(const population *pop, entity *joe,
   last updated: 1 Feb 2002
  **********************************************************************/
 
-void ga_chromosome_double_from_bytes(const population *pop, entity *joe, byte *bytes)
+GAULFUNC void ga_chromosome_double_from_bytes(const population *pop, entity *joe, gaulbyte *bytes)
   {
 
   if (!pop) die("Null pointer to population structure passed.");
@@ -586,7 +628,7 @@ void ga_chromosome_double_from_bytes(const population *pop, entity *joe, byte *b
   last updated: 19 Aug 2002
  **********************************************************************/
 
-char *ga_chromosome_double_to_string(
+GAULFUNC char *ga_chromosome_double_to_string(
                               const population *pop, const entity *joe,
                               char *text, size_t *textlen)
   {
@@ -597,7 +639,7 @@ char *ga_chromosome_double_to_string(
   if (!pop) die("Null pointer to population structure passed.");
   if (!joe) die("Null pointer to entity structure passed.");
 
-  if (!text || *textlen < 10 * pop->len_chromosomes * pop->num_chromosomes)
+  if (!text || (int) *textlen < 10 * pop->len_chromosomes * pop->num_chromosomes)
     {
     *textlen = 10 * pop->len_chromosomes * pop->num_chromosomes;
     text = s_realloc(text, sizeof(char) * *textlen);
@@ -615,7 +657,7 @@ char *ga_chromosome_double_to_string(
       {
       if (*textlen-k<8)
         {
-	*textlen *= 2;	/* FIXME: This isn't intelligent. */
+	*textlen *= 2;	/* Double allocation. */
         text = s_realloc(text, sizeof(char) * *textlen);
         }
 
@@ -624,7 +666,7 @@ char *ga_chromosome_double_to_string(
 
       if (l == -1)
         {	/* Truncation occured. */
-	*textlen *= 2;	/* FIXME: This isn't intelligent. */
+	*textlen *= 2;	/* Double allocation. */
         text = s_realloc(text, sizeof(char) * *textlen);
         l = snprintf(&(text[k]), *textlen-k, "%f ",
                        ((double *)joe->chromosome[i])[j]);
@@ -651,7 +693,7 @@ char *ga_chromosome_double_to_string(
   last updated: 16/06/01
  **********************************************************************/
 
-boolean ga_chromosome_char_allocate(const population *pop, entity *embryo)
+GAULFUNC boolean ga_chromosome_char_allocate(population *pop, entity *embryo)
   {
   int		i;		/* Loop variable over all chromosomes */
 
@@ -661,8 +703,10 @@ boolean ga_chromosome_char_allocate(const population *pop, entity *embryo)
   if (embryo->chromosome!=NULL)
     die("This entity already contains chromosomes.");
 
-  embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(char *));
-  embryo->chromosome[0] = s_malloc(pop->num_chromosomes*pop->len_chromosomes*sizeof(char));
+  if ( !(embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(char *))) )
+    die("Unable to allocate memory");
+  if ( !(embryo->chromosome[0] = s_malloc(pop->num_chromosomes*pop->len_chromosomes*sizeof(char))) )
+    die("Unable to allocate memory");
 
   for (i=1; i<pop->num_chromosomes; i++)
     {
@@ -675,13 +719,13 @@ boolean ga_chromosome_char_allocate(const population *pop, entity *embryo)
 
 /**********************************************************************
   ga_chromosome_char_deallocate()
-  synopsis:	Dellocate the chromosomes for an entity.
+  synopsis:	Deallocate the chromosomes for an entity.
   parameters:
   return:
   last updated: 16/06/01
  **********************************************************************/
 
-void ga_chromosome_char_deallocate(const population *pop, entity *corpse)
+GAULFUNC void ga_chromosome_char_deallocate(population *pop, entity *corpse)
   {
 
   if (!pop) die("Null pointer to population structure passed.");
@@ -708,7 +752,7 @@ void ga_chromosome_char_deallocate(const population *pop, entity *corpse)
   last updated: 19 Mar 2002
  **********************************************************************/
 
-void ga_chromosome_char_replicate( const population *pop,
+GAULFUNC void ga_chromosome_char_replicate( const population *pop,
                                    entity *parent, entity *child,
                                    const int chromosomeid )
   {
@@ -733,8 +777,8 @@ void ga_chromosome_char_replicate( const population *pop,
   last updated: 1 Feb 2002
  **********************************************************************/
 
-unsigned int ga_chromosome_char_to_bytes(const population *pop, entity *joe,
-                                    byte **bytes, unsigned int *max_bytes)
+GAULFUNC unsigned int ga_chromosome_char_to_bytes(const population *pop, entity *joe,
+                                    gaulbyte **bytes, unsigned int *max_bytes)
   {
   int		num_bytes;	/* Actual size of genes. */
 
@@ -745,13 +789,13 @@ unsigned int ga_chromosome_char_to_bytes(const population *pop, entity *joe,
 
   if (!joe->chromosome)
     {
-    *bytes = (byte *)"\0";
+    *bytes = (gaulbyte *)"\0";
     return 0;
     }
 
   num_bytes = pop->len_chromosomes * pop->num_chromosomes * sizeof(char);
 
-  *bytes = (byte *)joe->chromosome[0];
+  *bytes = (gaulbyte *)joe->chromosome[0];
 
   return num_bytes;
   }
@@ -766,7 +810,7 @@ unsigned int ga_chromosome_char_to_bytes(const population *pop, entity *joe,
   last updated: 1 Feb 2002
  **********************************************************************/
 
-void ga_chromosome_char_from_bytes(const population *pop, entity *joe, byte *bytes)
+GAULFUNC void ga_chromosome_char_from_bytes(const population *pop, entity *joe, gaulbyte *bytes)
   {
 
   if (!pop) die("Null pointer to population structure passed.");
@@ -789,7 +833,7 @@ void ga_chromosome_char_from_bytes(const population *pop, entity *joe, byte *byt
   last updated: 19 Aug 2002
  **********************************************************************/
 
-char *ga_chromosome_char_to_string(
+GAULFUNC char *ga_chromosome_char_to_string(
                               const population *pop, const entity *joe,
                               char *text, size_t *textlen)
   {
@@ -799,7 +843,7 @@ char *ga_chromosome_char_to_string(
   if (!pop) die("Null pointer to population structure passed.");
   if (!joe) die("Null pointer to entity structure passed.");
 
-  if (*textlen < pop->len_chromosomes * pop->num_chromosomes + 1)
+  if ( (int) *textlen < pop->len_chromosomes * pop->num_chromosomes + 1 )
     {
     *textlen = pop->len_chromosomes * pop->num_chromosomes + 1;
     text = s_realloc(text, sizeof(char) * *textlen);
@@ -833,7 +877,7 @@ char *ga_chromosome_char_to_string(
   last updated: 30/06/01
  **********************************************************************/
 
-boolean ga_chromosome_bitstring_allocate(const population *pop, entity *embryo)
+GAULFUNC boolean ga_chromosome_bitstring_allocate(population *pop, entity *embryo)
   {
   int		i;		/* Loop variable over all chromosomes */
 
@@ -843,7 +887,8 @@ boolean ga_chromosome_bitstring_allocate(const population *pop, entity *embryo)
   if (embryo->chromosome!=NULL)
     die("This entity already contains chromosomes.");
 
-  embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(byte *));
+  if ( !(embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(gaulbyte *))) )
+    die("Unable to allocate memory");
 
   for (i=0; i<pop->num_chromosomes; i++)
     embryo->chromosome[i] = ga_bit_new(pop->len_chromosomes);
@@ -854,13 +899,13 @@ boolean ga_chromosome_bitstring_allocate(const population *pop, entity *embryo)
 
 /**********************************************************************
   ga_chromosome_bitstring_deallocate()
-  synopsis:	Dellocate the chromosomes for an entity.
+  synopsis:	Deallocate the chromosomes for an entity.
   parameters:
   return:
   last updated: 30/06/01
  **********************************************************************/
 
-void ga_chromosome_bitstring_deallocate(const population *pop, entity *corpse)
+GAULFUNC void ga_chromosome_bitstring_deallocate(population *pop, entity *corpse)
   {
   int		i;		/* Loop variable over all chromosomes */
 
@@ -888,7 +933,7 @@ void ga_chromosome_bitstring_deallocate(const population *pop, entity *corpse)
   last updated: 30/06/01
  **********************************************************************/
 
-void ga_chromosome_bitstring_replicate( const population *pop,
+GAULFUNC void ga_chromosome_bitstring_replicate( const population *pop,
                                       entity *parent, entity *child,
                                       const int chromosomeid )
   {
@@ -913,9 +958,9 @@ void ga_chromosome_bitstring_replicate( const population *pop,
   last updated: 30/06/01
  **********************************************************************/
 
-unsigned int ga_chromosome_bitstring_to_bytes(const population *pop,
+GAULFUNC unsigned int ga_chromosome_bitstring_to_bytes(const population *pop,
                                     entity *joe,
-                                     byte **bytes, unsigned int *max_bytes)
+                                     gaulbyte **bytes, unsigned int *max_bytes)
   {
   int		num_bytes;	/* Actual size of genes. */
   int		i;		/* Loop variable over all chromosomes */
@@ -923,18 +968,18 @@ unsigned int ga_chromosome_bitstring_to_bytes(const population *pop,
   if (!pop) die("Null pointer to population structure passed.");
   if (!joe) die("Null pointer to entity structure passed.");
 
-  num_bytes = ga_bit_sizeof(pop->len_chromosomes) * pop->num_chromosomes;
+  num_bytes = (int) ga_bit_sizeof(pop->len_chromosomes) * pop->num_chromosomes;
 
-  if (num_bytes>*max_bytes)
+  if (num_bytes>(int)*max_bytes)
     {
     *max_bytes = num_bytes;
-    *bytes = s_realloc(*bytes, *max_bytes*sizeof(byte));
-    /* sizeof(byte) should always be 1 */
+    *bytes = s_realloc(*bytes, *max_bytes*sizeof(gaulbyte));
+    /* sizeof(gaulbyte) should always be 1 */
     }
 
   if (!joe->chromosome)
     {
-    *bytes = (byte *)0;
+    *bytes = (gaulbyte *)0;
     return 0;
     }
 
@@ -958,9 +1003,9 @@ unsigned int ga_chromosome_bitstring_to_bytes(const population *pop,
   last updated: 13/06/01
  **********************************************************************/
 
-void ga_chromosome_bitstring_from_bytes( const population *pop,
+GAULFUNC void ga_chromosome_bitstring_from_bytes( const population *pop,
                                          entity *joe,
-                                         byte *bytes )
+                                         gaulbyte *bytes )
   {
   int		i;		/* Loop variable over all chromosomes */
 
@@ -988,7 +1033,7 @@ void ga_chromosome_bitstring_from_bytes( const population *pop,
   last updated: 19 Aug 2002
  **********************************************************************/
 
-char *ga_chromosome_bitstring_to_string(
+GAULFUNC char *ga_chromosome_bitstring_to_string(
                               const population *pop, const entity *joe,
                               char *text, size_t *textlen)
   {
@@ -998,7 +1043,7 @@ char *ga_chromosome_bitstring_to_string(
   if (!pop) die("Null pointer to population structure passed.");
   if (!joe) die("Null pointer to entity structure passed.");
 
-  if (!text || *textlen < pop->len_chromosomes * pop->num_chromosomes + 1)
+  if (!text || (int) *textlen < pop->len_chromosomes * pop->num_chromosomes + 1)
     {
     *textlen = pop->len_chromosomes * pop->num_chromosomes + 1;
     text = s_realloc(text, sizeof(char) * *textlen);
@@ -1024,4 +1069,178 @@ char *ga_chromosome_bitstring_to_string(
   }
 
 
+/**********************************************************************
+  ga_chromosome_list_allocate()
+  synopsis:	Allocate the chromosomes for an entity.  Initial
+		contents are set to null.
+  parameters:
+  return:
+  last updated: 05 Oct 2004
+ **********************************************************************/
+
+GAULFUNC boolean ga_chromosome_list_allocate(population *pop, entity *embryo)
+  {
+  int		i;		/* Loop variable over all chromosomes */
+
+  if (!pop) die("Null pointer to population structure passed.");
+  if (!embryo) die("Null pointer to entity structure passed.");
+
+  if (embryo->chromosome!=NULL)
+    die("This entity already contains chromosomes.");
+
+  if ( !(embryo->chromosome = s_malloc(pop->num_chromosomes*sizeof(DLList *))) )
+    die("Unable to allocate memory");
+
+  for (i=0; i<pop->num_chromosomes; i++)
+    embryo->chromosome[i] = NULL;
+
+  return TRUE;
+  }
+
+
+/**********************************************************************
+  ga_chromosome_list_deallocate()
+  synopsis:	Deallocate the chromosomes for an entity.
+  parameters:
+  return:
+  last updated: 05 Oct 2004
+ **********************************************************************/
+
+GAULFUNC void ga_chromosome_list_deallocate(population *pop, entity *corpse)
+  {
+  int		i;		/* Loop variable over all chromosomes */
+
+  if (!pop) die("Null pointer to population structure passed.");
+  if (!corpse) die("Null pointer to entity structure passed.");
+
+  if (corpse->chromosome==NULL)
+    die("This entity already contains no chromosomes.");
+
+  for (i=0; i<pop->num_chromosomes; i++)
+    dlink_free_all(corpse->chromosome[i]);
+
+  s_free(corpse->chromosome);
+  corpse->chromosome=NULL;
+
+  return;
+  }
+
+
+/**********************************************************************
+  ga_chromosome_list_replicate()
+  synopsis:	Duplicate a chromosome exactly.
+		Currently unimplemented at present.
+  parameters:
+  return:
+  last updated: 05 Oct 2004
+ **********************************************************************/
+
+GAULFUNC void ga_chromosome_list_replicate( const population *pop,
+                                      entity *parent, entity *child,
+                                      const int chromosomeid )
+  {
+
+  if (!pop) die("Null pointer to population structure passed.");
+  if (!parent || !child) die("Null pointer to entity structure passed.");
+  if (!parent->chromosome || !child->chromosome) die("Entity has no chromsomes.");
+
+  child->chromosome[chromosomeid] = dlink_clone(
+                parent->chromosome[chromosomeid] );
+
+  return;
+  }
+
+
+/**********************************************************************
+  ga_chromosome_list_to_bytes()
+  synopsis:	Convert to contiguous form.
+		Currently unimplemented at present.
+		FIXME: Need a user-defined callback to implement this
+		according to contents of the list.
+  parameters:
+  return:
+  last updated: 05 Oct 2004
+ **********************************************************************/
+
+GAULFUNC unsigned int ga_chromosome_list_to_bytes(const population *pop,
+                                    entity *joe,
+                                     gaulbyte **bytes, unsigned int *max_bytes)
+  {
+  int		num_bytes=0;	/* Actual size of genes. */
+
+  if (!pop) die("Null pointer to population structure passed.");
+  if (!joe) die("Null pointer to entity structure passed.");
+
+  die("ga_chromosome_list_to_bytes() is not implemented.");
+
+  /* Avoid compiler warnings. */
+  **bytes = 0;
+  *max_bytes = 0;
+
+  return num_bytes;
+  }
+
+
+/**********************************************************************
+  ga_chromosome_list_from_bytes()
+  synopsis:	Convert from contiguous form.  In this case, a trivial
+		process.
+		Currently unimplemented at present.
+		FIXME: Need a user-defined callback to implement this
+		according to contents of the list.
+  parameters:
+  return:
+  last updated: 05 Oct 2004
+ **********************************************************************/
+
+GAULFUNC void ga_chromosome_list_from_bytes( const population *pop,
+                                         entity *joe,
+                                         gaulbyte *bytes )
+  {
+
+  if (!pop) die("Null pointer to population structure passed.");
+  if (!joe) die("Null pointer to entity structure passed.");
+
+  if (!joe->chromosome) die("Entity has no chromsomes.");
+
+  die("ga_chromosome_list_from_bytes() is not implemented.");
+
+  /* Avoid compiler warning. */
+  *bytes = 0;
+
+  return;
+  }
+
+
+/**********************************************************************
+  ga_chromosome_list_to_string()
+  synopsis:	Convert to human readable form.
+		Currently unimplemented at present.
+		FIXME: Need a user-defined callback to implement this
+		according to contents of the list.
+  parameters:
+  return:
+  last updated: 05 Oct 2004
+ **********************************************************************/
+
+GAULFUNC char *ga_chromosome_list_to_string(
+                              const population *pop, const entity *joe,
+                              char *text, size_t *textlen)
+  {
+
+  if (!pop) die("Null pointer to population structure passed.");
+  if (!joe) die("Null pointer to entity structure passed.");
+
+  if (!text || *textlen < 14)
+    {
+    *textlen = 14;
+    text = s_realloc(text, sizeof(char) * *textlen);
+    }
+
+  strncpy(text, "<unavailable>", 14);
+
+  return text;
+  }
+
+
 
