@@ -3,7 +3,7 @@
  **********************************************************************
 
   ga_mutate - Genetic algorithm mutation operators.
-  Copyright ©2000-2004, Stewart Adcock <stewart@linux-domain.com>
+  Copyright ©2000-2006, Stewart Adcock <stewart@linux-domain.com>
   All rights reserved.
 
   The latest version of this program should be available at:
@@ -40,10 +40,10 @@
 		allele is cycled.
   parameters:
   return:
-  last updated: 01/09/00
+  last updated: 17 Feb 2005
  **********************************************************************/
 
-void ga_mutate_integer_singlepoint_drift( population *pop,
+GAULFUNC void ga_mutate_integer_singlepoint_drift( population *pop,
                                           entity *father, entity *son )
   {
   int		i;		/* Loop variable over all chromosomes */
@@ -79,8 +79,10 @@ void ga_mutate_integer_singlepoint_drift( population *pop,
  */
   ((int *)son->chromosome[chromo])[point] += dir;
 
-  if (((int *)son->chromosome[chromo])[point]==RAND_MAX) ((int *)son->chromosome[chromo])[point]=0;
-  if (((int *)son->chromosome[chromo])[point]==-1) ((int *)son->chromosome[chromo])[point]=RAND_MAX-1;
+  if (((int *)son->chromosome[chromo])[point] > pop->allele_max_integer)
+    ((int *)son->chromosome[chromo])[point] = pop->allele_min_integer;
+  if (((int *)son->chromosome[chromo])[point] < pop->allele_min_integer)
+    ((int *)son->chromosome[chromo])[point] = pop->allele_max_integer;
 
   return;
   }
@@ -92,10 +94,10 @@ void ga_mutate_integer_singlepoint_drift( population *pop,
 		allele is randomized.
   parameters:
   return:
-  last updated: 01/09/00
+  last updated: 17 Feb 2005
  **********************************************************************/
 
-void ga_mutate_integer_singlepoint_randomize( population *pop,
+GAULFUNC void ga_mutate_integer_singlepoint_randomize( population *pop,
                                               entity *father, entity *son )
   {
   int		i;		/* Loop variable over all chromosomes */
@@ -123,7 +125,7 @@ void ga_mutate_integer_singlepoint_randomize( population *pop,
       }
     }
 
-  ((int *)son->chromosome[chromo])[point] = (int) random_int(RAND_MAX);
+  ((int *)son->chromosome[chromo])[point] = (int) random_int_range(pop->allele_min_integer,pop->allele_max_integer+1);
 
   return;
   }
@@ -135,10 +137,10 @@ void ga_mutate_integer_singlepoint_randomize( population *pop,
 		to the more common 'bit-drift' mutation.
   parameters:
   return:
-  last updated: 21/07/00
+  last updated: 17 Feb 2005
  **********************************************************************/
 
-void ga_mutate_integer_multipoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_integer_multipoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
@@ -161,14 +163,14 @@ void ga_mutate_integer_multipoint(population *pop, entity *father, entity *son)
     {
     for (point=0; point<pop->len_chromosomes; point++)
       {
-      if (random_boolean_prob(GA_MULTI_BIT_CHANCE))
+      if (random_boolean_prob(pop->allele_mutation_prob))
         {
         ((int *)son->chromosome[chromo])[point] += dir;
 
-        if (((int *)son->chromosome[chromo])[point]==RAND_MAX)
-          ((int *)son->chromosome[chromo])[point]=0;
-        if (((int *)son->chromosome[chromo])[point]==-1)
-          ((int *)son->chromosome[chromo])[point]=RAND_MAX-1;
+        if (((int *)son->chromosome[chromo])[point] > pop->allele_max_integer)
+          ((int *)son->chromosome[chromo])[point] = pop->allele_min_integer;
+        if (((int *)son->chromosome[chromo])[point] < pop->allele_min_integer)
+          ((int *)son->chromosome[chromo])[point] = pop->allele_max_integer;
         }
       }
     }
@@ -184,10 +186,10 @@ void ga_mutate_integer_multipoint(population *pop, entity *father, entity *son)
 		remaining the same.
   parameters:
   return:
-  last updated: 28 May 2002
+  last updated: 17 Feb 2005
  **********************************************************************/
 
-void ga_mutate_integer_allpoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_integer_allpoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
@@ -214,16 +216,16 @@ void ga_mutate_integer_allpoint(population *pop, entity *father, entity *son)
         case (1):
           (((int *)son->chromosome[chromo])[point])++;
 
-          if (((int *)son->chromosome[chromo])[point]==RAND_MAX)
-            ((int *)son->chromosome[chromo])[point]=0;
+          if (((int *)son->chromosome[chromo])[point] > pop->allele_max_integer)
+            ((int *)son->chromosome[chromo])[point] = pop->allele_min_integer;
 
           break;
 
         case (2):
           (((int *)son->chromosome[chromo])[point])--;
 
-          if (((int *)son->chromosome[chromo])[point]==-1)
-            ((int *)son->chromosome[chromo])[point]=RAND_MAX-1;
+          if (((int *)son->chromosome[chromo])[point] < pop->allele_min_integer)
+            ((int *)son->chromosome[chromo])[point] = pop->allele_max_integer;
 
           break;
 
@@ -247,7 +249,7 @@ void ga_mutate_integer_allpoint(population *pop, entity *father, entity *son)
   last updated: 31/05/01
  **********************************************************************/
 
-void ga_mutate_boolean_singlepoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_boolean_singlepoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
@@ -285,10 +287,10 @@ void ga_mutate_boolean_singlepoint(population *pop, entity *father, entity *son)
   synopsis:	Cause a number of mutation events.
   parameters:
   return:
-  last updated: 31/05/01
+  last updated: 16 Feb 2005
  **********************************************************************/
 
-void ga_mutate_boolean_multipoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_boolean_multipoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
@@ -310,7 +312,7 @@ void ga_mutate_boolean_multipoint(population *pop, entity *father, entity *son)
     {
     for (point=0; point<pop->len_chromosomes; point++)
       {
-      if (random_boolean_prob(GA_MULTI_BIT_CHANCE))
+      if (random_boolean_prob(pop->allele_mutation_prob))
         {
         ((boolean *)son->chromosome[chromo])[point] = !((boolean *)son->chromosome[chromo])[point];
         }
@@ -330,7 +332,7 @@ void ga_mutate_boolean_multipoint(population *pop, entity *father, entity *son)
   last updated: 16/06/01
  **********************************************************************/
 
-void ga_mutate_char_singlepoint_drift( population *pop,
+GAULFUNC void ga_mutate_char_singlepoint_drift( population *pop,
                                        entity *father, entity *son )
   {
   int		i;		/* Loop variable over all chromosomes */
@@ -364,7 +366,7 @@ void ga_mutate_char_singlepoint_drift( population *pop,
 /*
  * Mutate by tweaking a single allele.
  */
-  ((char *)son->chromosome[chromo])[point] += dir;
+  ((char *)son->chromosome[chromo])[point] += (char)dir;
 
 /* Don't need these because char's **should** wrap safely.
   if (((char *)son->chromosome[chromo])[point]>CHAR_MAX)
@@ -387,7 +389,7 @@ void ga_mutate_char_singlepoint_drift( population *pop,
   last updated: 03 Jun 2002
  **********************************************************************/
 
-void ga_mutate_char_allpoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_char_allpoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
@@ -442,7 +444,7 @@ void ga_mutate_char_allpoint(population *pop, entity *father, entity *son)
   last updated: 16/06/01
  **********************************************************************/
 
-void ga_mutate_char_singlepoint_randomize( population *pop,
+GAULFUNC void ga_mutate_char_singlepoint_randomize( population *pop,
                                            entity *father, entity *son )
   {
   int		i;		/* Loop variable over all chromosomes */
@@ -470,7 +472,7 @@ void ga_mutate_char_singlepoint_randomize( population *pop,
       }
     }
 
-  ((char *)son->chromosome[chromo])[point] = (int) random_int(CHAR_MAX-CHAR_MIN)+CHAR_MIN;
+  ((char *)son->chromosome[chromo])[point] = (char) random_int(CHAR_MAX-CHAR_MIN)+CHAR_MIN;
 
   return;
   }
@@ -482,10 +484,10 @@ void ga_mutate_char_singlepoint_randomize( population *pop,
 		to the more common 'bit-drift' mutation.
   parameters:
   return:
-  last updated: 16/06/01
+  last updated: 16 Feb 2005
  **********************************************************************/
 
-void ga_mutate_char_multipoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_char_multipoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
@@ -508,9 +510,9 @@ void ga_mutate_char_multipoint(population *pop, entity *father, entity *son)
     {
     for (point=0; point<pop->len_chromosomes; point++)
       {
-      if (random_boolean_prob(GA_MULTI_BIT_CHANCE))
+      if (random_boolean_prob(pop->allele_mutation_prob))
         {
-        ((char *)son->chromosome[chromo])[point] += dir;
+        ((char *)son->chromosome[chromo])[point] += (char)dir;
 
 /* Don't need these because char's **should** wrap safely.
         if (((char *)son->chromosome[chromo])[point]>CHAR_MAX)
@@ -535,7 +537,7 @@ void ga_mutate_char_multipoint(population *pop, entity *father, entity *son)
   last updated: 16/06/01
  **********************************************************************/
 
-void ga_mutate_printable_singlepoint_drift( population *pop,
+GAULFUNC void ga_mutate_printable_singlepoint_drift( population *pop,
                                        entity *father, entity *son )
   {
   int		i;		/* Loop variable over all chromosomes */
@@ -569,7 +571,7 @@ void ga_mutate_printable_singlepoint_drift( population *pop,
 /*
  * Mutate by tweaking a single allele.
  */
-  ((char *)son->chromosome[chromo])[point] += dir;
+  ((char *)son->chromosome[chromo])[point] += (char)dir;
 
   if (((char *)son->chromosome[chromo])[point]>'~')
     ((char *)son->chromosome[chromo])[point]=' ';
@@ -589,7 +591,7 @@ void ga_mutate_printable_singlepoint_drift( population *pop,
   last updated: 16/06/01
  **********************************************************************/
 
-void ga_mutate_printable_singlepoint_randomize( population *pop,
+GAULFUNC void ga_mutate_printable_singlepoint_randomize( population *pop,
                                            entity *father, entity *son )
   {
   int		i;		/* Loop variable over all chromosomes */
@@ -617,7 +619,7 @@ void ga_mutate_printable_singlepoint_randomize( population *pop,
       }
     }
 
-  ((char *)son->chromosome[chromo])[point] = (int) random_int('~'-' ')+' ';
+  ((char *)son->chromosome[chromo])[point] = (char) random_int('~'-' ')+' ';
 
   return;
   }
@@ -629,10 +631,10 @@ void ga_mutate_printable_singlepoint_randomize( population *pop,
 		to the more common 'bit-drift' mutation.
   parameters:
   return:
-  last updated: 16/06/01
+  last updated: 16 Feb 2005
  **********************************************************************/
 
-void ga_mutate_printable_multipoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_printable_multipoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
@@ -655,9 +657,9 @@ void ga_mutate_printable_multipoint(population *pop, entity *father, entity *son
     {
     for (point=0; point<pop->len_chromosomes; point++)
       {
-      if (random_boolean_prob(GA_MULTI_BIT_CHANCE))
+      if (random_boolean_prob(pop->allele_mutation_prob))
         {
-        ((char *)son->chromosome[chromo])[point] += dir;
+        ((char *)son->chromosome[chromo])[point] += (char)dir;
 
         if (((char *)son->chromosome[chromo])[point]>'~')
           ((char *)son->chromosome[chromo])[point]=' ';
@@ -681,7 +683,7 @@ void ga_mutate_printable_multipoint(population *pop, entity *father, entity *son
   last updated: 10 Sep 2003
  **********************************************************************/
 
-void ga_mutate_printable_allpoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_printable_allpoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
@@ -741,7 +743,7 @@ void ga_mutate_printable_allpoint(population *pop, entity *father, entity *son)
   last updated: 30/06/01
  **********************************************************************/
 
-void ga_mutate_bitstring_singlepoint( population *pop,
+GAULFUNC void ga_mutate_bitstring_singlepoint( population *pop,
                                     entity *father, entity *son )
   {
   int		i;		/* Loop variable over all chromosomes */
@@ -782,10 +784,10 @@ void ga_mutate_bitstring_singlepoint( population *pop,
   synopsis:	Cause a number of mutation events.
   parameters:
   return:
-  last updated: 10 Sep 2003
+  last updated: 16 Feb 2005
  **********************************************************************/
 
-void ga_mutate_bitstring_multipoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_bitstring_multipoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
@@ -807,7 +809,7 @@ void ga_mutate_bitstring_multipoint(population *pop, entity *father, entity *son
     {
     for (point=0; point<pop->len_chromosomes; point++)
       {
-      if (random_boolean_prob(GA_MULTI_BIT_CHANCE))
+      if (random_boolean_prob(pop->allele_mutation_prob))
         {
         ga_bit_invert(son->chromosome[chromo],point);
         }
@@ -824,10 +826,10 @@ void ga_mutate_bitstring_multipoint(population *pop, entity *father, entity *son
 		allele is adjusted.  (Unit Gaussian distribution.)
   parameters:
   return:
-  last updated: 19 Apr 2002
+  last updated: 17 Feb 2005
  **********************************************************************/
 
-void ga_mutate_double_singlepoint_drift( population *pop,
+GAULFUNC void ga_mutate_double_singlepoint_drift( population *pop,
                                           entity *father, entity *son )
   {
   int		i;		/* Loop variable over all chromosomes */
@@ -863,8 +865,10 @@ void ga_mutate_double_singlepoint_drift( population *pop,
  */
   ((double *)son->chromosome[chromo])[point] += amount;
 
-  if (((double *)son->chromosome[chromo])[point]>DBL_MAX-1.0) ((double *)son->chromosome[chromo])[point]=DBL_MIN+1.0;
-  if (((double *)son->chromosome[chromo])[point]<DBL_MIN+1.0) ((double *)son->chromosome[chromo])[point]=DBL_MAX-1.0;
+  if (((double *)son->chromosome[chromo])[point] > pop->allele_max_double)
+    ((double *)son->chromosome[chromo])[point] -= (pop->allele_max_double-pop->allele_min_double);
+  if (((double *)son->chromosome[chromo])[point] < pop->allele_min_double)
+    ((double *)son->chromosome[chromo])[point] += (pop->allele_max_double-pop->allele_min_double);
 
   return;
   }
@@ -879,7 +883,7 @@ void ga_mutate_double_singlepoint_drift( population *pop,
   last updated: 19 Apr 2002
  **********************************************************************/
 
-void ga_mutate_double_singlepoint_randomize( population *pop,
+GAULFUNC void ga_mutate_double_singlepoint_randomize( population *pop,
                                               entity *father, entity *son )
   {
   int		i;		/* Loop variable over all chromosomes */
@@ -920,15 +924,14 @@ void ga_mutate_double_singlepoint_randomize( population *pop,
 		(Unit Gaussian distribution.)
   parameters:
   return:
-  last updated: 10 Sep 2003
+  last updated: 17 Feb 2005
  **********************************************************************/
 
-void ga_mutate_double_multipoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_double_multipoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
   int		point;		/* Index of allele to mutate */
-  double	amount=random_unit_gaussian();	/* The amount of drift. */
 
 /* Checks */
   if (!father || !son) die("Null pointer to entity structure passed");
@@ -946,9 +949,14 @@ void ga_mutate_double_multipoint(population *pop, entity *father, entity *son)
     {
     for (point=0; point<pop->len_chromosomes; point++)
       {
-      if (random_boolean_prob(GA_MULTI_BIT_CHANCE))
+      if (random_boolean_prob(pop->allele_mutation_prob))
         {
-        ((double *)son->chromosome[chromo])[point] += amount;
+        ((double *)son->chromosome[chromo])[point] += random_unit_gaussian();
+
+        if (((double *)son->chromosome[chromo])[point] > pop->allele_max_double)
+          ((double *)son->chromosome[chromo])[point] -= (pop->allele_max_double-pop->allele_min_double);
+        if (((double *)son->chromosome[chromo])[point] < pop->allele_min_double)
+          ((double *)son->chromosome[chromo])[point] += (pop->allele_max_double-pop->allele_min_double);
         }
       }
     }
@@ -964,10 +972,10 @@ void ga_mutate_double_multipoint(population *pop, entity *father, entity *son)
 		(Unit Gaussian distribution.)
   parameters:
   return:
-  last updated: 10 Sep 2003
+  last updated: 17 Feb 2005
  **********************************************************************/
 
-void ga_mutate_double_allpoint(population *pop, entity *father, entity *son)
+GAULFUNC void ga_mutate_double_allpoint(population *pop, entity *father, entity *son)
   {
   int		i;		/* Loop variable over all chromosomes */
   int		chromo;		/* Index of chromosome to mutate */
@@ -983,13 +991,18 @@ void ga_mutate_double_allpoint(population *pop, entity *father, entity *son)
     }
 
 /*
- * Mutate by incrementing or decrementing alleles.
+ * Mutate by adjusting all alleles.
  */
   for (chromo=0; chromo<pop->num_chromosomes; chromo++)
     {
     for (point=0; point<pop->len_chromosomes; point++)
       {
       (((double *)son->chromosome[chromo])[point]) += random_unit_gaussian();
+
+      if (((double *)son->chromosome[chromo])[point] > pop->allele_max_double)
+        ((double *)son->chromosome[chromo])[point] -= (pop->allele_max_double-pop->allele_min_double);
+      if (((double *)son->chromosome[chromo])[point] < pop->allele_min_double)
+        ((double *)son->chromosome[chromo])[point] += (pop->allele_max_double-pop->allele_min_double);
       }
     }
 
