@@ -3,7 +3,7 @@
  **********************************************************************
 
   ga_utility - High-level genetic algorithm routines.
-  Copyright ©2000-2003, Stewart Adcock <stewart@linux-domain.com>
+  Copyright ©2000-2006, Stewart Adcock <stewart@linux-domain.com>
   All rights reserved.
 
   The latest version of this program should be available at:
@@ -29,50 +29,53 @@
 
   To do:	Population/entity iterator functions.
 		On-line and off-line performance summaries.
-		The ga_genesis_XXX() functions are bloated junk which duplicate loads of code and are a complete mess.  These need sorting out!
 
  **********************************************************************/
 
-#include "gaul/ga_utility.h"
+#include "gaul.h"
 
 
 /**********************************************************************
   ga_diagnostics()
-  synopsis:	Diagnostics.
+  synopsis:	Display diagnostics message.
   parameters:	none
   return:	none
-  last updated:	14 Jun 2004
+  last updated:	17 Feb 2005
  **********************************************************************/
 
-void ga_diagnostics(void)
+GAULFUNC void ga_diagnostics(void)
   {
   int	num_pops;	/* Number of populations defined, or -1 for no table. */
 
   printf("=== GA utility library =======================================\n");
-  printf("Version:                   %s\n", GA_VERSION_STRING);
-  printf("Build date:                %s\n", GA_BUILD_DATE_STRING);
+  printf("Version:                     %s\n", GA_VERSION_STRING);
+  printf("Build date:                  %s\n", GA_BUILD_DATE_STRING);
   printf("Compilation machine characteristics:\n%s\n", GA_UNAME_STRING);
   printf("--- Constants ------------------------------------------------\n");
-  printf("GA_DEBUG:                  %d\n", GA_DEBUG);
-  printf("GA_BOLTZMANN_FACTOR:       %e\n", GA_BOLTZMANN_FACTOR);
-  printf("GA_MIN_FITNESS:            %e\n", GA_MIN_FITNESS);
-  printf("GA_MULTI_BIT_CHANCE:       %f\n", GA_MULTI_BIT_CHANCE);
-  printf("BYTEBITS:                  %d\n", BYTEBITS);
+  printf("GA_DEBUG:                    %d\n", GA_DEBUG);
+  printf("GA_BOLTZMANN_FACTOR:         %e\n", GA_BOLTZMANN_FACTOR);
+  printf("GA_MIN_FITNESS:              %e\n", GA_MIN_FITNESS);
+  printf("BYTEBITS:                    %d\n", BYTEBITS);
+  printf("--- Defaults -------------------------------------------------\n");
+  printf("GA_DEFAULT_CROSSOVER_RATIO:  %f\n", GA_DEFAULT_CROSSOVER_RATIO);
+  printf("GA_DEFAULT_MUTATION_RATIO:   %f\n", GA_DEFAULT_MUTATION_RATIO);
+  printf("GA_DEFAULT_MIGRATION_RATIO:  %f\n", GA_DEFAULT_MIGRATION_RATIO);
+  printf("GA_DEFAULT_ALLELE_MUTATION_PROB: %f\n", GA_DEFAULT_ALLELE_MUTATION_PROB);
   printf("--- Data structures ------------------------------------------\n");
-  printf("structure                  sizeof\n");
-  printf("population                 %lu\n", (unsigned long) sizeof(population));
-  printf("entity                     %lu\n", (unsigned long) sizeof(entity));
-  printf("byte                       %lu\n", (unsigned long) sizeof(byte));
+  printf("structure                    sizeof\n");
+  printf("population                   %lu\n", (unsigned long) sizeof(population));
+  printf("entity                       %lu\n", (unsigned long) sizeof(entity));
+  printf("gaulbyte                     %lu\n", (unsigned long) sizeof(gaulbyte));
   printf("--- Current variables ----------------------------------------\n");
   num_pops = ga_get_num_populations();
   if (num_pops==-1)
     {
-    printf("Population table:          undefined\n");
+    printf("Population table:            undefined\n");
     }
   else
     {
-    printf("Population table:          defined\n");
-    printf("Size:                      %d\n", num_pops);
+    printf("Population table:            defined\n");
+    printf("Size:                        %d\n", num_pops);
     }
   printf("==============================================================\n");
 
@@ -87,7 +90,7 @@ void ga_diagnostics(void)
   last updated:	06 Apr 2003
  **********************************************************************/
 
-int ga_get_major_version( void )
+GAULFUNC int ga_get_major_version( void )
   {
 
   return GA_MAJOR_VERSION;
@@ -102,7 +105,7 @@ int ga_get_major_version( void )
   last updated:	06 Apr 2003
  **********************************************************************/
 
-int ga_get_minor_version( void )
+GAULFUNC int ga_get_minor_version( void )
   {
 
   return GA_MINOR_VERSION;
@@ -117,7 +120,7 @@ int ga_get_minor_version( void )
   last updated:	06 Apr 2003
  **********************************************************************/
 
-int ga_get_patch_version( void )
+GAULFUNC int ga_get_patch_version( void )
   {
 
   return GA_PATCH_VERSION;
@@ -133,10 +136,10 @@ int ga_get_patch_version( void )
 		Integer-valued chromsomes.
   parameters:
   return:	population, or NULL on failure.
-  last updated:	17 Jan 2003
+  last updated:	17 Feb 2005
  **********************************************************************/
 
-population *ga_genesis_integer(	const int		population_size,
+GAULFUNC population *ga_genesis_integer(	const int		population_size,
 			const int		num_chromo,
 			const int		len_chromo,
 			GAgeneration_hook	generation_hook,
@@ -202,6 +205,7 @@ population *ga_genesis_integer(	const int		population_size,
 /*
  * Seed the population.
  */
+#if 0
   if (seed==NULL)
     {
     plog(LOG_VERBOSE, "Entity seed function not defined.  Genesis can not occur.  Continuing anyway.");
@@ -211,6 +215,7 @@ population *ga_genesis_integer(	const int		population_size,
     ga_population_seed(pop);
     plog(LOG_VERBOSE, "Genesis has occured!");
     }
+#endif
 
   return pop;
   }
@@ -233,7 +238,7 @@ population *ga_genesis_integer(	const int		population_size,
 
 #ifndef COMPILE_DEPRECATED_FUNCTIONS
 
-population *ga_genesis(	const int		population_size,
+GAULFUNC population *ga_genesis(	const int		population_size,
 			const int		num_chromo,
 			const int		len_chromo,
 			GAgeneration_hook	generation_hook,
@@ -260,7 +265,7 @@ population *ga_genesis(	const int		population_size,
                              userdata );
   }
 
-population *ga_genesis_int(	const int		population_size,
+GAULFUNC population *ga_genesis_int(	const int		population_size,
 			const int		num_chromo,
 			const int		len_chromo,
 			GAgeneration_hook	generation_hook,
@@ -297,10 +302,10 @@ population *ga_genesis_int(	const int		population_size,
 		Character-valued chromosomes.
   parameters:
   return:	population, or NULL on failure.
-  last updated:	13 Feb 2002
+  last updated:	17 Feb 2005
  **********************************************************************/
 
-population *ga_genesis_char(	const int		population_size,
+GAULFUNC population *ga_genesis_char(	const int		population_size,
 			const int		num_chromo,
 			const int		len_chromo,
 			GAgeneration_hook	generation_hook,
@@ -366,6 +371,7 @@ population *ga_genesis_char(	const int		population_size,
 /*
  * Seed the population.
  */
+#if 0
   if (seed==NULL)
     {
     plog(LOG_VERBOSE, "Entity seed function not defined.  Genesis can not occur.  Continuing anyway.");
@@ -375,6 +381,7 @@ population *ga_genesis_char(	const int		population_size,
     ga_population_seed(pop);
     plog(LOG_VERBOSE, "Genesis has occured!");
     }
+#endif
 
   return pop;
   }
@@ -388,10 +395,10 @@ population *ga_genesis_char(	const int		population_size,
 		Boolean-valued chromosomes.
   parameters:
   return:	population, or NULL on failure.
-  last updated:	13 Feb 2002
+  last updated:	17 Feb 2005
  **********************************************************************/
 
-population *ga_genesis_boolean(	const int		population_size,
+GAULFUNC population *ga_genesis_boolean(	const int		population_size,
 			const int		num_chromo,
 			const int		len_chromo,
 			GAgeneration_hook	generation_hook,
@@ -457,6 +464,7 @@ population *ga_genesis_boolean(	const int		population_size,
 /*
  * Seed the population.
  */
+#if 0
   if (seed==NULL)
     {
     plog(LOG_VERBOSE, "Entity seed function not defined.  Genesis can not occur.  Continuing anyway.");
@@ -466,6 +474,7 @@ population *ga_genesis_boolean(	const int		population_size,
     ga_population_seed(pop);
     plog(LOG_VERBOSE, "Genesis has occured!");
     }
+#endif
 
   return pop;
   }
@@ -479,10 +488,10 @@ population *ga_genesis_boolean(	const int		population_size,
 		Double precision real-valued chromosomes.
   parameters:
   return:	population, or NULL on failure.
-  last updated:	13 Feb 2002
+  last updated:	17 Feb 2005
  **********************************************************************/
 
-population *ga_genesis_double(	const int		population_size,
+GAULFUNC population *ga_genesis_double(	const int		population_size,
 			const int		num_chromo,
 			const int		len_chromo,
 			GAgeneration_hook	generation_hook,
@@ -548,6 +557,7 @@ population *ga_genesis_double(	const int		population_size,
 /*
  * Seed the population.
  */
+#if 0
   if (seed==NULL)
     {
     plog(LOG_VERBOSE, "Entity seed function not defined.  Genesis can not occur.  Continuing anyway.");
@@ -557,6 +567,7 @@ population *ga_genesis_double(	const int		population_size,
     ga_population_seed(pop);
     plog(LOG_VERBOSE, "Genesis has occured!");
     }
+#endif
 
   return pop;
   }
@@ -570,10 +581,10 @@ population *ga_genesis_double(	const int		population_size,
 		Bitstring-valued chromosomes.
   parameters:
   return:	population, or NULL on failure.
-  last updated:	13 Feb 2002
+  last updated:	17 Feb 2005
  **********************************************************************/
 
-population *ga_genesis_bitstring(	const int		population_size,
+GAULFUNC population *ga_genesis_bitstring(	const int		population_size,
 			const int		num_chromo,
 			const int		len_chromo,
 			GAgeneration_hook	generation_hook,
@@ -639,6 +650,7 @@ population *ga_genesis_bitstring(	const int		population_size,
 /*
  * Seed the population.
  */
+#if 0
   if (seed==NULL)
     {
     plog(LOG_VERBOSE, "Entity seed function not defined.  Genesis can not occur.  Continuing anyway.");
@@ -648,6 +660,7 @@ population *ga_genesis_bitstring(	const int		population_size,
     ga_population_seed(pop);
     plog(LOG_VERBOSE, "Genesis has occured!");
     }
+#endif
 
   return pop;
   }
@@ -670,7 +683,7 @@ population *ga_genesis_bitstring(	const int		population_size,
 
 #ifndef COMPILE_DEPRECATED_FUNCTIONS
 
-entity *ga_allele_search(	population	*pop,
+GAULFUNC entity *ga_allele_search(	population	*pop,
 				const int	chromosomeid,
 				const int	point,
 				const int 	min_val,
@@ -755,14 +768,16 @@ entity *ga_allele_search(	population	*pop,
   synopsis:	Dump some statistics about a population.
   parameters:	population	*pop
   return:	none
-  last updated:	16/06/01
+  last updated:	17 Feb 2005
  **********************************************************************/
 
-void ga_population_dump(population	*pop)
+GAULFUNC void ga_population_dump(population	*pop)
   {
   printf("Population id %d\n", (int) ga_get_population_id(pop));
   printf("Max size %d Stable size %d Current size %d\n", pop->max_size, pop->stable_size, pop->size);
   printf("Crossover %f Mutation %f Migration %f\n", pop->crossover_ratio, pop->mutation_ratio, pop->migration_ratio);
+  printf("Allele mutation prob %f\n", pop->allele_mutation_prob);
+  printf("Allele ranges %d - %d %f - %f\n", pop->allele_min_integer, pop->allele_max_integer, pop->allele_min_double, pop->allele_max_double);
   printf("Chromosome length %d count %d\n", pop->len_chromosomes, pop->num_chromosomes);
   printf("Best fitness %f\n", pop->entity_iarray[0]->fitness);
 
@@ -778,11 +793,17 @@ void ga_population_dump(population	*pop)
   last updated:	24 Dec 2002
  **********************************************************************/
 
-void ga_entity_dump(population *pop, entity *john)
+GAULFUNC void ga_entity_dump(population *pop, entity *john)
   {
-  printf("Entity id %d rank %d\n", ga_get_entity_id(pop, john), ga_get_entity_rank(pop, john));
-  printf("Fitness %f\n", john->fitness);
-  printf("data <%s> data0 <%s> chromo <%s> chromo0 <%s>\n", john->data?"defined":"undefined", john->data!=NULL&&((SLList *)john->data)->data!=NULL?"defined":"undefined", john->chromosome?"defined":"undefined", john->chromosome!=NULL&&john->chromosome[0]!=NULL?"defined":"undefined");
+  printf( "Entity id %d rank %d\n",
+          ga_get_entity_id(pop, john),
+          ga_get_entity_rank(pop, john) );
+  printf( "Fitness %f\n", john->fitness );
+  printf( "data <%s> data0 <%s> chromo <%s> chromo0 <%s>\n",
+          john->data?"defined":"undefined",
+          john->data!=NULL&&((SLList *)john->data)->data!=NULL?"defined":"undefined",
+          john->chromosome?"defined":"undefined",
+          john->chromosome!=NULL&&john->chromosome[0]!=NULL?"defined":"undefined" );
 
   return;
   }
