@@ -3,7 +3,7 @@
  **********************************************************************
 
   memory_util - Usage control wrapper around standard malloc() etc.
-  Copyright ©1999-2004, Stewart Adcock <stewart@linux-domain.com>
+  Copyright ©1999-2006, Stewart Adcock <stewart@linux-domain.com>
   All rights reserved.
 
   The latest version of this program should be available at:
@@ -125,7 +125,7 @@ typedef struct mem_record_t
 
 THREAD_LOCK_DEFINE_STATIC(memory_mem_record_chunk);
 THREAD_LOCK_DEFINE_STATIC(memory_memtree);
-#if USE_OPENMP == 1
+#ifdef USE_OPENMP
 static boolean memory_openmp_initialised = FALSE;
 #endif
 
@@ -161,7 +161,7 @@ static int	node_count=0;		/* counting tree nodes for debugging. */
 void memory_init_openmp(void)
   {
 
-#if USE_OPENMP == 1
+#ifdef USE_OPENMP
   if (memory_openmp_initialised == FALSE)
     {
     omp_init_lock(&memory_mem_record_chunk);
@@ -210,7 +210,7 @@ static mem_record *mem_record_new(void)
   THREAD_LOCK(memory_mem_record_chunk);
   if (!mem_record_chunk) mem_record_chunk = mem_chunk_new(sizeof(mem_record), 1024);
 
-  mr = mem_chunk_alloc(mem_record_chunk);
+  mr = (mem_record *)mem_chunk_alloc(mem_record_chunk);
   THREAD_UNLOCK(memory_mem_record_chunk);
 
   num_mem++;
@@ -560,8 +560,8 @@ static void free_padded(mem_record *mr)
  **********************************************************************/
 
 void *s_alloc_debug(memory_alloc_type type, size_t memsize, int numvars, void *mptr,
-		char *name, char *file, int line,
-		char *label)
+		const char *name, const char *file, const int line,
+		const char *label)
   {
   mem_record	*j=NULL;			/* Current memory record. */
   size_t	size_low, size_high;		/* Padding amount. */
@@ -721,7 +721,7 @@ printf("Realloc mptr = %p j = %p j->mptr = %p\n", mptr, j, j->mptr);
       case (MEMORY_STRNDUP):
         /* ...and strndup. */
         len = strlen(mptr);
-        if (memsize > len) memsize=len;
+        if (memsize > (size_t)len) memsize=(size_t)len;
         if ( !(mtemp=malloc(memsize+size_low+size_high)) )
           {
           dief("Strdup of %lu bytes failed at func=%s file=%s line=%d\n",
@@ -755,12 +755,12 @@ printf("Realloc mptr = %p j = %p j->mptr = %p\n", mptr, j, j->mptr);
     j->pad_hs=size_high;
     if(size_high)
       {
-      for (k=0; k<size_high; k++) j->pad_high[k]=rand_high;
+      for (k=0; size_high>=(size_t)k; k++) j->pad_high[k]=rand_high;
       pad_mptr_high(j);
       }
     if(size_low)
       {
-      for (k=0; k<size_low; k++) j->pad_low[k]=rand_low;
+      for (k=0; size_low>=(size_t)k; k++) j->pad_low[k]=rand_low;
       pad_mptr_low(j);
       }
 
@@ -855,12 +855,12 @@ printf("REPLACING j->mptr %p with mtemp %p j %p\n", j->mptr, mtemp, j);
       j->pad_hs=size_high;
       if (size_high)
         {
-        for (k=0; k<size_high; k++) j->pad_high[k]=rand_high;
+        for (k=0; size_high>=(size_t)k; k++) j->pad_high[k]=rand_high;
         pad_mptr_high(j);
         }
       if (size_low)
         {
-        for (k=0; k<size_low; k++) j->pad_low[k]=rand_low;
+        for (k=0; size_low>=(size_t)k; k++) j->pad_low[k]=rand_low;
         pad_mptr_low(j);
         }
 
@@ -880,12 +880,12 @@ printf("INSERT (2) mtemp = %p j = %p j->mptr = %p\n", mtemp, j, j->mptr);
 
       if(j->pad_hs)
         { 
-        for (k=0; k<size_high; k++) j->pad_high[k]=rand_high;
+        for (k=0; size_high>=(size_t)k; k++) j->pad_high[k]=rand_high;
         pad_mptr_high(j);
         }
       if (j->pad_ls)
         {
-        for (k=0; k<size_low; k++) j->pad_low[k]=rand_low;
+        for (k=0; size_low>=(size_t)k; k++) j->pad_low[k]=rand_low;
         pad_mptr_low(j);
         }
       }
@@ -918,7 +918,7 @@ printf("INSERT (2) mtemp = %p j = %p j->mptr = %p\n", mtemp, j, j->mptr);
   last updated: 17/07/00
  **********************************************************************/
 
-void *s_free_debug(void *mptr, char *name, char *file, int line)
+void *s_free_debug(void *mptr, const char *name, const char *file, const int line)
   {
   mem_record	*j;	/* Index of current memory record. */
 
@@ -1437,7 +1437,7 @@ void memory_set_padding(int i)
  **********************************************************************/
 
 void *s_malloc_safe(	size_t size,
-			char *funcname, char *filename, int linenum)
+			const char *funcname, const char *filename, const int linenum)
   {
   void	*ptr;	/* Pointer to new memory */
    
@@ -1473,7 +1473,7 @@ void *s_malloc_safe(	size_t size,
  **********************************************************************/
 
 void *s_calloc_safe(	size_t num, size_t size,
-			char *funcname, char *filename, int linenum)
+			const char *funcname, const char *filename, const int linenum)
   {
   void	*ptr;	/* Pointer to new memory */
 
@@ -1509,7 +1509,7 @@ void *s_calloc_safe(	size_t num, size_t size,
  **********************************************************************/
 
 void *s_realloc_safe(	void *oldptr, size_t size,
-			char *funcname, char *filename, int linenum)
+			const char *funcname, const char *filename, const int linenum)
   {
   void	*ptr;	/* Pointer to new memory */
    
@@ -1538,7 +1538,7 @@ void *s_realloc_safe(	void *oldptr, size_t size,
  **********************************************************************/
 
 char *s_strdup_safe(	const char *src,
-			char *funcname, char *filename, int linenum)
+			const char *funcname, const char *filename, const int linenum)
   {
   void		*dest;	/* Pointer to new string */
   size_t	len;	/* String length */
@@ -1586,7 +1586,7 @@ char *s_strdup_safe(	const char *src,
  **********************************************************************/
 
 char *s_strndup_safe(	const char *src, size_t length,
-			char *funcname, char *filename, int linenum )
+			const char *funcname, const char *filename, const int linenum )
   {
   void		*dest;	/* Pointer to new string */
   size_t	len;	/* String length */
@@ -1636,10 +1636,13 @@ char *s_strndup_safe(	const char *src, size_t length,
 		check the return value.
   parameters:
   return:
-  last updated:	12/01/01
+  last updated:	10/02/05
  **********************************************************************/
 
-void s_free_safe(void *ptr, char *funcname, char *filename, int linenum)
+void s_free_safe(void *ptr,
+                 const char *funcname,
+                 const char *filename,
+                 const int linenum)
   {
   memory_count_free++;
 
